﻿using System.Security.Authentication;
using System.Security.Claims;
using IdentityModel;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using XF.Domain.Dependency;
using XF.Domain.Extensions;

namespace XF.Domain.Authentications
{
    public class DefaultSessionContext : ISessionContext, IScopeDependency
    {
        private HttpContext _context;
        private readonly IChangeSessionProvider _changeSessionProvider;
        private readonly IServiceProvider _serviceProvider;
        public DefaultSessionContext(IHttpContextAccessor httpContextAccessor, IServiceProvider serviceProvider)
        {
            _serviceProvider = serviceProvider;
            var httpContext = httpContextAccessor.HttpContext;
            if (httpContext is null)
            {
                _context = new DefaultHttpContext();
                return;
            }

            _context = httpContext;
            if (_context.Items.Any(m => m.Key == nameof(DefaultSessionContext))) return;
            _context.Items.Add(nameof(DefaultSessionContext), nameof(IChangeSessionProvider));
            _changeSessionProvider = httpContextAccessor.HttpContext.RequestServices.GetService<IChangeSessionProvider>();
        }

        /// <summary>
        /// Id of current tenant or null for host
        /// </summary>
        public string? UserId { get => _context.User.FindFirstValue(ClaimTypes.NameIdentifier); init { } }

        /// <summary>
        /// Id of current user or throw Exception for guest
        /// </summary>
        /// <exception cref="AuthenticationException"></exception>
        public string RequiredUserId => UserId ?? throw new ArgumentNullException();

        public string? UserName { get => _context.User.FindFirstValue(AppClaimTypes.UserDisplayName); init { } }

        public string? Phone { get => _context.User.FindFirstValue(JwtClaimTypes.PhoneNumber); init { } }

        /// <summary>
        /// Roles
        /// 不能使用 JwtClaimTypes.Role 来获取Role
        /// 在 JwtBearerHandler 处理 JWT 令牌时，它会使用 Microsoft.AspNetCore.Authentication.JwtBearer 自动转换 Claim 名称。
        /// </summary>
        // public string[] Roles { get => _context.User.Claims.Where(d => d.Type == JwtClaimTypes.Role).Select(d => d.Value).ToArray(); init { } }
        public string[] Roles { get => _context.User.Claims.Where(d => d.Type == ClaimTypes.Role).Select(d => d.Value).ToArray(); init { } }

        public string? OrgId { get => _context.User.FindFirstValue(AppClaimTypes.DepartmentId); init { } }
        public string? OrgName { get => _context.User.FindFirstValue(AppClaimTypes.DepartmentName); init { } }

        public int OrgLevel { get => _context.User.FindIntValue(AppClaimTypes.DepartmentLevel); init { } }
        public string? OrgAreaCode { get => _context.User.FindFirstValue(AppClaimTypes.DepartmentAreaCode); init { } }
        public bool OrgIsCenter { get => _context.User.FindBoolValue(AppClaimTypes.DepartmentIsCenter); init { } }

        /// <summary>
        /// 部门行政区划名称
        /// </summary>
        public string? OrgAreaName { get => _context.User.FindFirstValue(AppClaimTypes.DepartmentAreaName); init { } }

        public string? AreaId { get => _context.User.FindFirstValue(AppClaimTypes.AreaId); init { } }

        public string RequiredOrgId => OrgId ?? throw new ArgumentNullException();
        
        public string? ClientId { get => _context.User.FindFirstValue(JwtClaimTypes.ClientId); init { } }

        /// <summary>
        /// 工号
        /// </summary>
        public string? StaffNo { get => _context.User.FindFirstValue(AppClaimTypes.StaffNo); init { } }

        /// <summary>
        /// 第三方平台用户唯一标识
        /// 例如: 微信的OpenId
        /// </summary>
        public string? OpenId { get => _context.User.FindFirstValue(AppClaimTypes.OpenId); init { } }

        public void ChangeSession(string id)
        {
            if (_changeSessionProvider == null)
            {
                _context = _serviceProvider.GetService<IChangeSessionProvider>().ChangeSessionByUserId(id, _context);
                return;
            }
            _changeSessionProvider.ChangeSessionByUserId(id, _context);
        }
    }

    public static class ClaimsPrincipalExtensions
    {
        public static int FindIntValue(this ClaimsPrincipal claims, string claimType)
        {
            var value = claims.FindFirstValue(claimType);
            return int.TryParse(value, out int result) ? result : default;
        }

        public static bool FindBoolValue(this ClaimsPrincipal claims, string claimType)
        {
            var value = claims.FindFirstValue(claimType);
            return bool.TryParse(value, out bool result) ? result : default;
        }
    }
}